from pyecore.resources import global_registry
from .types import getEClassifier, eClassifiers
from .types import name, nsURI, nsPrefix, eClass
from .types import Type, VisualType, ImportType, CompositeType, PointerType, QuantityType, ParameterType, StateVariableType, DynamicsType, ArgumentType, ExpressionType, HTMLType, JSONType, TextType, URLType, PointType, ArrayType, CompositeVisualType, ConnectionType, SimpleType, ImageType, SimpleArrayType, MetadataType

from pygeppetto.model import DomainModel, Tag
from pygeppetto.model.variables import Variable
from pygeppetto.model.values import ArrayValue, Pointer, Quantity, Argument, Text, VisualValue, JSON, VisualGroup, Point, Expression, AArrayValue, Composite, Image, URL, HTML, Dynamics

from . import types
from .. import model


__all__ = ['Type', 'VisualType', 'ImportType', 'CompositeType', 'PointerType', 'QuantityType', 'ParameterType', 'StateVariableType', 'DynamicsType', 'ArgumentType', 'ExpressionType',
           'HTMLType', 'JSONType', 'TextType', 'URLType', 'PointType', 'ArrayType', 'CompositeVisualType', 'ConnectionType', 'SimpleType', 'ImageType', 'SimpleArrayType', 'MetadataType']

eSubpackages = []
eSuperPackage = model
types.eSubpackages = eSubpackages
types.eSuperPackage = eSuperPackage


otherClassifiers = []

for classif in otherClassifiers:
    eClassifiers[classif.name] = classif
    classif.ePackage = eClass

for classif in eClassifiers.values():
    eClass.eClassifiers.append(classif.eClass)

for subpack in eSubpackages:
    eClass.eSubpackages.append(subpack.eClass)

register_packages = [types] + eSubpackages
for pack in register_packages:
    global_registry[pack.nsURI] = pack



from pyecore.resources import global_registry
from .variables import getEClassifier, eClassifiers
from .variables import name, nsURI, nsPrefix, eClass
from .variables import Variable, TypeToValueMap

from pygeppetto.model import Tag
from pygeppetto.model.values import Value, Point
from pygeppetto.model.types import Type

from . import variables
from .. import model


__all__ = ['Variable', 'TypeToValueMap']

eSubpackages = []
eSuperPackage = model
variables.eSubpackages = eSubpackages
variables.eSuperPackage = eSuperPackage


otherClassifiers = []

for classif in otherClassifiers:
    eClassifiers[classif.name] = classif
    classif.ePackage = eClass

for classif in eClassifiers.values():
    eClass.eClassifiers.append(classif.eClass)

for subpack in eSubpackages:
    eClass.eSubpackages.append(subpack.eClass)

register_packages = [variables] + eSubpackages
for pack in register_packages:
    global_registry[pack.nsURI] = pack



from pygeppetto.model.datasources import Query


def query_check(query: Query, types=()) -> bool:
    """
    https://github.com/openworm/org.geppetto.core/blob/master/src/main/java/org/geppetto/core/datasources/QueryChecker.java)
    :return:
    """
    if len(query.matchingCriteria) == 0:
        return True

    all_types = set()
    all_types.update(types)

    for criterion in query.matchingCriteria:
        for type_to_match in criterion.type:
            if not type_to_match in all_types:
                for type_ in all_types:
                    if type_.extends_type(type_to_match):
                        return True
    return False

